package cn.kk.spring_simple_operation.service.impl;

import cn.kk.spring_simple_operation.entity.PhotoProgress;
import cn.kk.spring_simple_operation.mapper.PhotoProgressMapper;
import cn.kk.spring_simple_operation.model.dto.SkuSiteDTO;
import cn.kk.spring_simple_operation.service.PhotoProgressService;
import cn.kk.spring_simple_operation.service.VisualDesignLogService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 图片协同系统 服务实现类
 * </p>
 *
 * @author guokunkun
 * @since 2024-12-25
 */
@Service
public class PhotoProgressServiceImpl extends ServiceImpl<PhotoProgressMapper, PhotoProgress> implements PhotoProgressService {

	@Resource
	private VisualDesignLogService visualDesignLogService;

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void setUploadStatusIsDeleted(List<SkuSiteDTO> list) {
		List<PhotoProgress> listPhotoProgress = baseMapper.getListBySku(list.stream().map(SkuSiteDTO::getSku).collect(Collectors.toList()));
		List<Long> ids = listPhotoProgress.stream()
				.filter(t -> t.getSimpleVideoUploadStatus() != 5)
				.map(PhotoProgress::getId)
				.collect(Collectors.toList());

		update(Wrappers.<PhotoProgress>lambdaUpdate().in(PhotoProgress::getId, ids)
				.set(PhotoProgress::getSimpleVideoUploadStatus, 5)
		);
		visualDesignLogService.addBatchLog("photo_progress", "根据文件将sku的视频上传后台状态设置为已删除", ids);
	}
}
